using System;
using System.Collections.Generic;
using System.Linq;
using System.Transactions;
using System.Workflow.Activities;
using C1WorkflowDemo.DataTypes;
using Composite.Actions;
using Composite.Data;
using Composite.Transactions;
using Composite.Workflow;


namespace C1WorkflowDemo.Workflows
{
    [AllowPersistingWorkflow(WorkflowPersistingType.Idle)]
    public sealed partial class AddNewProductWorkflow : Composite.Workflow.Activities.FormsWorkflow
    {
        private bool _step1Valid = false;
        private bool _step2Valid = false;

        public AddNewProductWorkflow()
        {
            InitializeComponent();
        }




        private void IsStep1Valid(object sender, ConditionalEventArgs e)
        {
            e.Result = _step1Valid;
        }




        private void IsStep2Valid(object sender, ConditionalEventArgs e)
        {
            e.Result = _step2Valid;
        }




        private void CreateNewProductCategory(object sender, ConditionalEventArgs e)
        {
            IProduct newProduct = this.GetBinding<IProduct>("NewProduct");

            e.Result = newProduct.ProductCategoryId == Guid.Empty;
        }



        // Create new product data object and pre set Id and category
        private void step1CodeActivity_ExecuteCode(object sender, EventArgs e)
        {
            if (this.BindingExist("NewProduct") == false)
            {
                IProduct newProduct = DataFacade.BuildNew<IProduct>();
                newProduct.Id = Guid.NewGuid();

                if (this.EntityToken is DataEntityToken && ((DataEntityToken)this.EntityToken).InterfaceType == typeof(IProductCategory))
                {
                    DataEntityToken dataEntityToken = (DataEntityToken)this.EntityToken;
                    newProduct.ProductCategoryId = dataEntityToken.DataSourceId.GetKeyValue<Guid>();
                }

                this.Bindings.Add("NewProduct", newProduct);
            }

            if (this.BindingExist("ProductCategories") == false)
            {
                Dictionary<Guid, string> productCategories =
                    (from pc in DataFacade.GetData<IProductCategory>()
                     select pc).ToDictionary(f => f.Id, f => f.Name);

                productCategories.Add(Guid.Empty, "New Product Category");

                this.Bindings.Add("ProductCategories", productCategories);
            }
        }



        // Validating step 1, price
        private void step1CodeActivity_Validation_ExecuteCode(object sender, EventArgs e)
        {
            _step1Valid = true;

            IProduct newProduct = this.GetBinding<IProduct>("NewProduct");

            if ((newProduct.Price % 0.05M) != 0.0M)
            {
                this.ShowFieldMessage("NewProduct.Price", "The price must be divisible with 0.05");

                _step1Valid = false;
            }
        }




        private void step2CodeActivity_ExecuteCode(object sender, EventArgs e)
        {
            if (this.BindingExist("NewProductCategory") == false)
            {
                IProductCategory newProductCategory = DataFacade.BuildNew<IProductCategory>();

                newProductCategory.Id = Guid.NewGuid();

                this.Bindings.Add("NewProductCategory", newProductCategory);
            }
        }



        // Validating new categury
        private void step2codeActivity_Validation_ExecuteCode(object sender, EventArgs e)
        {
            _step2Valid = true;

            IProductCategory newProductCategory = this.GetBinding<IProductCategory>("NewProductCategory");

            IProductCategory existingProductCategory =
                (from pc in DataFacade.GetData<IProductCategory>()
                 where pc.Name == newProductCategory.Name
                 select pc).FirstOrDefault();

            if (existingProductCategory != null)
            {
                _step2Valid = false;

                this.ShowFieldMessage("NewProductCategory.Name", "The product category name is in use");
            }
        }


        // The finish event
        private void finalizeCodeActivity_ExecuteCode(object sender, EventArgs e)
        {
            using (TransactionScope transactionScope = TransactionsFacade.CreateNewScope())
            {
                IProduct newProduct = this.GetBinding<IProduct>("NewProduct");

                IProductCategory newProductCategory = null;

                if (newProduct.ProductCategoryId == Guid.Empty)
                {
                    // "New Category" - save category to data layer
                    newProductCategory = this.GetBinding<IProductCategory>("NewProductCategory");
                    newProductCategory = DataFacade.AddNew<IProductCategory>(newProductCategory);
                    newProduct.ProductCategoryId = newProductCategory.Id;
                }

                // Save product to data layer
                newProduct = DataFacade.AddNew<IProduct>(newProduct);

                // Refresh tree structure - on new categories we refresh higher up
                if (newProductCategory != null)
                {
                    ParentTreeRefresher parentTreeRefresher = this.CreateParentTreeRefresher();
                    parentTreeRefresher.PostRefreshMesseges(newProductCategory.GetDataEntityToken());
                }
                else
                {
                    IProductCategory productCategory =
                        (from pc in DataFacade.GetData<IProductCategory>()
                         where pc.Id == newProduct.ProductCategoryId
                         select pc).Single();

                    SpecificTreeRefresher specificTreeRefresher = this.CreateSpecificTreeRefresher();
                    specificTreeRefresher.PostRefreshMesseges(productCategory.GetDataEntityToken());
                }

                // Start up the edit work flow
                this.ExecuteWorklow(newProduct.GetDataEntityToken(), typeof(EditProductWorkflow));

                transactionScope.Complete();
            }
        }

    }
}
